#  Copyright 2021 Google LLC
#  Copyright 2021 EPAM Systems
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""
Validate Manifest against R3 schemas operator.
"""

import logging

from airflow.utils import apply_defaults
from airflow.models import BaseOperator, Variable
from libs.context import Context
from libs.exceptions import EmptyManifestError
from libs.refresh_token import AirflowTokenRefresher
from libs.validation.validate_schema import SchemaValidator
from operators.mixins.ReceivingContextMixin import ReceivingContextMixin

logger = logging.getLogger()


class ValidateManifestSchemaOperator(BaseOperator, ReceivingContextMixin):
    """Operator to validate manifest against definition schemasR3."""

    ui_color = '#dad5ff'
    ui_fgcolor = '#000000'

    @apply_defaults
    def __init__(self, previous_task_id: str = None, *args, **kwargs):
        """Init base operator and obtain base urls from Airflow Variables."""
        super().__init__(*args, **kwargs)
        self.previous_task_id = previous_task_id
        self.schema_service_url = Variable.get('schema_service_url')

    def execute(self, context: dict):
        """Execute manifest validation then process it.

        Get a single manifest file or a list of them.
        If it is a list, calculate which range (slice) of manifest files must be processed and then
        process this range one by one.

        :param context: Airflow context
        :type context: dict
        """
        execution_context = context["dag_run"].conf["execution_context"]
        payload_context = Context.populate(execution_context)
        token_refresher = AirflowTokenRefresher()

        schema_validator = SchemaValidator(
            self.schema_service_url,
            token_refresher,
            payload_context
        )

        manifest_data = self._get_manifest_data(context, execution_context)
        logger.debug(f"Manifest data: {manifest_data}")

        if not manifest_data:
            raise EmptyManifestError(
                f"Data {context['dag_run'].conf} doesn't contain 'manifest field'")

        _ = schema_validator.validate_common_schema(manifest_data)
        valid_manifest_file = schema_validator.ensure_manifest_validity(manifest_data)
        return {"manifest": valid_manifest_file}
